unit ElevationColours;

{
Revision history:

2000 Apr 08  First version

Copyright  David J Taylor, Edinburgh
Web site:  http://www.davidtaylor.freeserve.co.uk
E-mail:    davidtaylor@writeme.com
}

interface

uses
  Graphics;

var
  colours: array [SmallInt] of TColor;

implementation

const
  min_interval = -2;
  max_interval =  8;

type
  intervals = min_interval..max_interval;

const
  // Array of heights in metres
  heights:
    array [intervals] of integer =
    (-60, -30, 0, 100, 200, 300, 500, 1000, 2000, 3000, 5000);

  // Array of colours at the heights given, will be interpolated
  shades: array [intervals] of TColor = (
    133 shl 16 + 219 shl 8 +  55,     //   -60
     95 shl 16 + 222 shl 8 +  98,     //   -30
     57 shl 16 + 225 shl 8 + 141,     //     0
    137 shl 16 + 237 shl 8 + 187,     //   100
    194 shl 16 + 245 shl 8 + 220,     //   200
    188 shl 16 + 216 shl 8 + 224,     //   300
    154 shl 16 + 196 shl 8 + 209,     //   500
    104 shl 16 + 176 shl 8 + 230,     //  1000
     79 shl 16 + 150 shl 8 + 170,     //  2000
    185 shl 16 + 149 shl 8 + 182,     //  3000
    clWhite                           //  5000
  );


procedure interpolate_colours (p, q: integer);
var
  r, g, b: integer;
  r1, g1, b1: integer;
  r2, g2, b2: integer;
  colour: integer;
begin
  // Get RGB values of starting and ending colours
  b1 := colours [p] shr 16;
  g1 := (colours [p] shr 8) and $FF;
  r1 := colours [p] and $FF;
  b2 := colours [q] shr 16;
  g2 := (colours [q] shr 8) and $FF;
  r2 := colours [q] and $FF;
  // Sweep the intermediate heights, using a linear interpolation
  // of colours between the defined height values
  for colour := (p + 1) to (q - 1) do
    begin
    r := ((q - colour) * r1 + (colour - p) * r2) div (q - p);
    g := ((q - colour) * g1 + (colour - p) * g2) div (q - p);
    b := ((q - colour) * b1 + (colour - p) * b2) div (q - p);
    colours [colour] := b shl 16 + g shl 8 + r;
    end;
end;


var
  band: min_interval..max_interval;
  h_previous: integer;
  height: integer;

initialization

  // Do the stuff below sea level to the minimum colour
  for height := -32768 to 0 do
    colours [height] := shades [min_interval];
  colours [-9999] := clBlack;
  // Sweep height pairs across remaining bands, i.e. those for which
  // contours are defined.  Define the height in the current band, and
  // set that at a preset colour, remembering that the heights are byte
  // swapped in the data file.  For all bands except the first one, perform
  // a smooth interpolation between the current height/colour and the
  // previous height/colour.  This improves the visual apperance of map
  // somewhat.
  h_previous := 0;          // Just to stop the compiler warning message...
  for band := min_interval to max_interval do
    begin
    height := heights [band];
    colours [height] := shades [band];
    if band <> min_interval then
      interpolate_colours (h_previous, height);
    h_previous := height;
    end;
  // Do the stuff above the max height - set it all white
  for height := h_previous to 32767 do
    colours [height] := shades [max_interval];

end.

