unit GSHHSdata;

{
2000 May 22  First version
2001 Feb 11  Version for Delphi 5 (no change required)

Copyright  David J Taylor, Edinburgh
Web site:  www.satsignal.net
E-mail:    davidtaylor@writeme.com
}

interface

const
  lvLand = 1;
  lvLake = 2;
  lvIslandInLake = 3;
  lvPonInIslandInLake = 4;

type
  // Global Self-consistant Hierarchical High-resolution Shorelines
  TGSHHS = record
    id: Longint;			// Unique polygon id number, starting at 0
    n: Longword;			// Number of points in this polygon
    level: Longword;                    // 1 land, 2 lake, 3 island_in_lake, 4 pond_in_island_in_lake
    west, east,
    south, north: Longint;              // min/max extent in micro-degrees
    area: Longword;                     // Area of polygon in 1/10 km^2
    greenwich: Word;                    // Greenwich is 1 if Greenwich is crossed
    source: Word;                       // 0 = CIA WDBII, 1 = WVS
  end;

type
  TGPOINT = record
    x: Longint;         // 32-bit longitude of a point in micro-degrees
    y: Longint;         // 32-bit latitude of a point in micro-degrees
  end;

procedure HeaderSwap (var hdr: TGSHHS);
procedure PointSwap (var xy: TGPOINT);


implementation

function Swap32 (L: Longword): Longword;
var
  r: Longword;
  b: integer;
begin
  r := 0;
  for b := 1 to 4 do
    begin
    r := r or (L and $000000FF);
    L := L shr 8;
    if b <> 4 then r := r shl 8;
    end;
  Result := r
end;


procedure HeaderSwap (var hdr: TGSHHS);
begin
  with hdr do
    begin
    id := Swap32 (id);
    n := Swap32 (n);
    level := Swap32 (level);
    west := Swap32 (west);
    east := Swap32 (east);
    north := Swap32 (north);
    south := Swap32 (south);
    area := Swap32 (area);
    greenwich := Swap (greenwich);
    source := Swap (source);
    end;
end;


procedure PointSwap (var xy: TGPOINT);
begin
  with xy do
    begin
    x := Swap32 (x);
    y := Swap32 (y);
    end;
end;

end.

